@@ -1,8 +1,8 @@
 from typing import Optional, TYPE_CHECKING
 
-from .box import get_safe_box, Box, SQUARE, ROUNDED
+from .box import Box, ROUNDED
 
-from .align import AlignValues
+from .align import AlignMethod
 from .jupyter import JupyterMixin
 from .measure import Measurement, measure_renderables
 from .padding import Padding, PaddingDimensions
@@ -39,13 +39,13 @@ class Panel(JupyterMixin):
         box: Box = ROUNDED,
         *,
         title: TextType = None,
-        title_align: AlignValues = "center",
+        title_align: AlignMethod = "center",
         safe_box: Optional[bool] = None,
         expand: bool = True,
         style: StyleType = "none",
         border_style: StyleType = "none",
         width: Optional[int] = None,
-        padding: PaddingDimensions = 0,
+        padding: PaddingDimensions = (0, 1),
     ) -> None:
         self.renderable = renderable
         self.box = box
@@ -65,12 +65,12 @@ class Panel(JupyterMixin):
         box: Box = ROUNDED,
         *,
         title: TextType = None,
-        title_align: AlignValues = "center",
+        title_align: AlignMethod = "center",
         safe_box: Optional[bool] = None,
         style: StyleType = "none",
         border_style: StyleType = "none",
         width: Optional[int] = None,
-        padding: PaddingDimensions = 0,
+        padding: PaddingDimensions = (0, 1),
     ):
         """An alternative constructor that sets expand=False."""
         return cls(
@@ -96,6 +96,7 @@ class Panel(JupyterMixin):
             )
             title_text.end = ""
             title_text.plain = title_text.plain.replace("\n", " ")
+            title_text.no_wrap = True
             title_text.expand_tabs()
             title_text.pad(1)
             return title_text
@@ -113,7 +114,7 @@ class Panel(JupyterMixin):
         width = options.max_width if self.width is None else self.width
 
         safe_box: bool = console.safe_box if self.safe_box is None else self.safe_box  # type: ignore
-        box = get_safe_box(self.box, console.legacy_windows) if safe_box else self.box
+        box = self.box.substitute(options, safe=safe_box)
 
         title_text = self._title
         if title_text is not None:
@@ -158,11 +159,17 @@ class Panel(JupyterMixin):
         _, right, _, left = Padding.unpack(self.padding)
         padding = left + right
         renderables = [self.renderable, _title] if _title else [self.renderable]
-        width = (
-            measure_renderables(console, renderables, max_width - padding - 2).maximum
-            + padding
-            + 2
-        )
+
+        if self.width is None:
+            width = (
+                measure_renderables(
+                    console, renderables, max_width - padding - 2
+                ).maximum
+                + padding
+                + 2
+            )
+        else:
+            width = self.width
         return Measurement(width, width)
 
 
