@@ -3,6 +3,7 @@ from typing import TYPE_CHECKING, Iterable, List, NamedTuple, Optional, Tuple, U
 
 from . import box, errors
 from ._loop import loop_first_last, loop_last
+from ._pick import pick_bool
 from ._ratio import ratio_distribute, ratio_reduce
 from .jupyter import JupyterMixin
 from .measure import Measurement
@@ -10,7 +11,6 @@ from .padding import Padding, PaddingDimensions
 from .protocol import is_renderable
 from .segment import Segment
 from .style import Style, StyleType
-from .styled import Styled
 from .text import Text, TextType
 
 if TYPE_CHECKING:
@@ -47,10 +47,17 @@ class Column:
     """str: How to justify text within the column ("left", "center", "right", or "full")"""
 
     overflow: "OverflowMethod" = "ellipsis"
+    """str: Overflow method."""
 
     width: Optional[int] = None
     """Optional[int]: Width of the column, or ``None`` (default) to auto calculate width."""
 
+    min_width: Optional[int] = None
+    """Optional[int]: Minimum width of column, or ``None`` for no minimum. Defaults to None."""
+
+    max_width: Optional[int] = None
+    """Optional[int]: Maximum width of column, or ``None`` for no maximum. Defaults to None."""
+
     ratio: Optional[int] = None
     """Optional[int]: Ratio to use when calculating column width, or ``None`` (default) to adapt to column contents."""
 
@@ -73,6 +80,17 @@ class Column:
         return self.ratio is not None
 
 
+@dataclass
+class Row:
+    """Information regarding a row."""
+
+    style: Optional[StyleType] = None
+    """Style to apply to row."""
+
+    end_section: bool = False
+    """Indicated end of section, which will force a line beneath the row."""
+
+
 class _Cell(NamedTuple):
     """A single cell in a table."""
 
@@ -90,6 +108,7 @@ class Table(JupyterMixin):
         title (Union[str, Text], optional): The title of the table rendered at the top. Defaults to None.
         caption (Union[str, Text], optional): The table caption rendered below. Defaults to None.
         width (int, optional): The width in characters of the table, or ``None`` to automatically fit. Defaults to None.
+        min_width (Optional[int], optional): The minimum width of the table, or ``None`` for no minimum. Defaults to None.
         box (box.Box, optional): One of the constants in box.py used to draw the edges (see :ref:`appendix_box`). Defaults to box.HEAVY_HEAD.
         safe_box (Optional[bool], optional): Disable box characters that don't display on windows legacy terminal with *raster* fonts. Defaults to True.
         padding (PaddingDimensions, optional): Padding for cells (top, right, bottom, left). Defaults to (0, 1).
@@ -113,6 +132,7 @@ class Table(JupyterMixin):
     """
 
     columns: List[Column]
+    rows: List[Row]
 
     def __init__(
         self,
@@ -120,6 +140,7 @@ class Table(JupyterMixin):
         title: TextType = None,
         caption: TextType = None,
         width: int = None,
+        min_width: int = None,
         box: Optional[box.Box] = box.HEAVY_HEAD,
         safe_box: Optional[bool] = None,
         padding: PaddingDimensions = (0, 1),
@@ -143,6 +164,7 @@ class Table(JupyterMixin):
     ) -> None:
 
         self.columns: List[Column] = []
+        self.rows: List[Row] = []
         append_column = self.columns.append
         for index, header in enumerate(headers):
             if isinstance(header, str):
@@ -154,6 +176,7 @@ class Table(JupyterMixin):
         self.title = title
         self.caption = caption
         self.width = width
+        self.min_width = min_width
         self.box = box
         self.safe_box = safe_box
         self._padding = Padding.unpack(padding)
@@ -170,10 +193,9 @@ class Table(JupyterMixin):
         self.footer_style = footer_style
         self.border_style = border_style
         self.title_style = title_style
-        self.caption_style = title_style
+        self.caption_style = caption_style
         self.title_justify = title_justify
         self.caption_justify = caption_justify
-        self._row_count = 0
         self.row_styles = list(row_styles or [])
 
     @classmethod
@@ -229,27 +251,26 @@ class Table(JupyterMixin):
     @property
     def row_count(self) -> int:
         """Get the current number of rows."""
-        return self._row_count
+        return len(self.rows)
 
-    def get_row_style(self, index: int) -> StyleType:
+    def get_row_style(self, console: "Console", index: int) -> StyleType:
         """Get the current row style."""
+        style = Style.null()
         if self.row_styles:
-            return self.row_styles[index % len(self.row_styles)]
-        return Style.empty()
+            style += console.get_style(self.row_styles[index % len(self.row_styles)])
+        row_style = self.rows[index].style
+        if row_style is not None:
+            style += console.get_style(row_style)
+        return style
 
     def __rich_measure__(self, console: "Console", max_width: int) -> Measurement:
         if self.width is not None:
             max_width = self.width
-
-        if self.box:
-            max_width -= len(self.columns) - 1
-            if self.show_edge:
-                max_width -= 2
-
         if max_width < 0:
             return Measurement(0, 0)
 
         extra_width = self._extra_width
+        max_width = sum(self._calculate_column_widths(console, max_width - extra_width))
         _measure_column = self._measure_column
 
         measurements = [
@@ -263,7 +284,9 @@ class Table(JupyterMixin):
             if (self.width is None)
             else self.width
         )
-        return Measurement(minimum_width, maximum_width)
+        measurement = Measurement(minimum_width, maximum_width)
+        measurement = measurement.clamp(self.min_width)
+        return measurement
 
     @property
     def padding(self) -> Tuple[int, int, int, int]:
@@ -287,6 +310,8 @@ class Table(JupyterMixin):
         justify: "JustifyMethod" = "left",
         overflow: "OverflowMethod" = "ellipsis",
         width: int = None,
+        min_width: int = None,
+        max_width: int = None,
         ratio: int = None,
         no_wrap: bool = False,
     ) -> None:
@@ -301,7 +326,9 @@ class Table(JupyterMixin):
             footer_style (Union[str, Style], optional): Style for the header. Defaults to "none".
             style (Union[str, Style], optional): Style for the column cells. Defaults to "none".
             justify (JustifyMethod, optional): Alignment for cells. Defaults to "left".
-            width (int, optional): A minimum width in characters. Defaults to None.
+            width (int, optional): Desired width of column in characters, or None to fit to contents. Defaults to None.
+            min_width (Optional[int], optional): Minimum width of column, or ``None`` for no minimum. Defaults to None.
+            max_width (Optional[int], optional): Maximum width of column, or ``None`` for no maximum. Defaults to None.
             ratio (int, optional): Flexible ratio for the column (requires ``Table.expand`` or ``Table.width``). Defaults to None.
             no_wrap (bool, optional): Set to ``True`` to disable wrapping of this column.
         """
@@ -320,13 +347,18 @@ class Table(JupyterMixin):
             justify=justify,
             overflow=overflow,
             width=width,
+            min_width=min_width,
+            max_width=max_width,
             ratio=ratio,
             no_wrap=no_wrap,
         )
         self.columns.append(column)
 
     def add_row(
-        self, *renderables: Optional["RenderableType"], style: StyleType = None
+        self,
+        *renderables: Optional["RenderableType"],
+        style: StyleType = None,
+        end_section: bool = False,
     ) -> None:
         """Add a row of renderables.
 
@@ -334,15 +366,14 @@ class Table(JupyterMixin):
             *renderables (None or renderable): Each cell in a row must be a renderable object (including str),
                 or ``None`` for a blank cell.
             style (StyleType, optional): An optional style to apply to the entire row. Defaults to None.
+            end_section (bool, optional): End a section and draw a line. Defaults to False.
 
         Raises:
             errors.NotRenderableError: If you add something that can't be rendered.
         """
 
         def add_cell(column: Column, renderable: "RenderableType") -> None:
-            column._cells.append(
-                renderable if style is None else Styled(renderable, style)
-            )
+            column._cells.append(renderable)
 
         cell_renderables: List[Optional["RenderableType"]] = list(renderables)
 
@@ -355,7 +386,7 @@ class Table(JupyterMixin):
         for index, renderable in enumerate(cell_renderables):
             if index == len(columns):
                 column = Column(_index=index)
-                for _ in range(self._row_count):
+                for _ in self.rows:
                     add_cell(column, Text(""))
                 self.columns.append(column)
             else:
@@ -368,7 +399,7 @@ class Table(JupyterMixin):
                 raise errors.NotRenderableError(
                     f"unable to render {type(renderable).__name__}; a string or other renderable object is required"
                 )
-        self._row_count += 1
+        self.rows.append(Row(style=style, end_section=end_section))
 
     def __rich_console__(
         self, console: "Console", options: "ConsoleOptions"
@@ -377,12 +408,10 @@ class Table(JupyterMixin):
         max_width = options.max_width
         if self.width is not None:
             max_width = self.width
-        if self.box:
-            max_width -= len(self.columns) - 1
-            if self.show_edge:
-                max_width -= 2
-        widths = self._calculate_column_widths(console, max_width)
-        table_width = sum(widths) + self._extra_width
+
+        extra_width = self._extra_width
+        widths = self._calculate_column_widths(console, max_width - extra_width)
+        table_width = sum(widths) + extra_width
 
         render_options = options.update(width=table_width)
 
@@ -418,6 +447,7 @@ class Table(JupyterMixin):
         ]
         widths = [_range.maximum or 1 for _range in width_ranges]
         get_padding_width = self._get_padding_width
+        extra_width = self._extra_width
 
         if self.expand:
             ratios = [col.ratio or 0 for col in columns if col.flexible]
@@ -441,7 +471,9 @@ class Table(JupyterMixin):
 
         if table_width > max_width:
             widths = self._collapse_widths(
-                widths, [not column.no_wrap for column in columns], max_width
+                widths,
+                [(column.width is None and not column.no_wrap) for column in columns],
+                max_width,
             )
             table_width = sum(widths)
 
@@ -457,8 +489,15 @@ class Table(JupyterMixin):
             ]
             widths = [_range.maximum or 1 for _range in width_ranges]
 
-        if table_width < max_width and self.expand:
-            pad_widths = ratio_distribute(max_width - table_width, widths)
+        if (table_width < max_width and self.expand) or (
+            self.min_width is not None and table_width < (self.min_width - extra_width)
+        ):
+            _max_width = (
+                max_width
+                if self.min_width is None
+                else min(self.min_width - extra_width, max_width)
+            )
+            pad_widths = ratio_distribute(_max_width - table_width, widths)
             widths = [_width + pad for _width, pad in zip(widths, pad_widths)]
 
         return widths
@@ -484,13 +523,10 @@ class Table(JupyterMixin):
                 max_column = max(
                     width for width, allow_wrap in zip(widths, wrapable) if allow_wrap
                 )
-                try:
-                    second_max_column = max(
-                        width if allow_wrap and width != max_column else 0
-                        for width, allow_wrap in zip(widths, wrapable)
-                    )
-                except ValueError:
-                    second_max_column = 0
+                second_max_column = max(
+                    width if allow_wrap and width != max_column else 0
+                    for width, allow_wrap in zip(widths, wrapable)
+                )
                 column_difference = max_column - second_max_column
                 ratios = [
                     (1 if (width == max_column and allow_wrap) else 0)
@@ -574,7 +610,7 @@ class Table(JupyterMixin):
             # Fixed width column
             return Measurement(
                 column.width + padding_width, column.width + padding_width
-            )
+            ).with_maximum(max_width)
         # Flexible column, we need to measure contents
         min_widths: List[int] = []
         max_widths: List[int] = []
@@ -586,10 +622,15 @@ class Table(JupyterMixin):
             append_min(_min)
             append_max(_max)
 
-        return Measurement(
+        measurement = Measurement(
             max(min_widths) if min_widths else 1,
             max(max_widths) if max_widths else max_width,
+        ).with_maximum(max_width)
+        measurement = measurement.clamp(
+            None if column.min_width is None else column.min_width + padding_width,
+            None if column.max_width is None else column.max_width + padding_width,
         )
+        return measurement
 
     def _render(
         self, console: "Console", options: "ConsoleOptions", widths: List[int]
@@ -597,17 +638,17 @@ class Table(JupyterMixin):
         table_style = console.get_style(self.style or "")
 
         border_style = table_style + console.get_style(self.border_style or "")
-        rows: List[Tuple[_Cell, ...]] = list(
-            zip(
-                *(
-                    self._get_cells(column_index, column)
-                    for column_index, column in enumerate(self.columns)
-                )
-            )
+        _column_cells = (
+            self._get_cells(column_index, column)
+            for column_index, column in enumerate(self.columns)
         )
-        safe_box: bool = console.safe_box if self.safe_box is None else self.safe_box  # type: ignore
+        row_cells: List[Tuple[_Cell, ...]] = list(zip(*_column_cells))
         _box = (
-            box.get_safe_box(self.box, console.legacy_windows) if safe_box else self.box
+            self.box.substitute(
+                options, safe=pick_bool(self.safe_box, console.safe_box)
+            )
+            if self.box
+            else None
         )
 
         # _box = self.box
@@ -648,18 +689,23 @@ class Table(JupyterMixin):
         get_row_style = self.get_row_style
         get_style = console.get_style
 
-        for index, (first, last, row) in enumerate(loop_first_last(rows)):
+        for index, (first, last, row_cell) in enumerate(loop_first_last(row_cells)):
             header_row = first and show_header
             footer_row = last and show_footer
+            row = (
+                self.rows[index - show_header]
+                if (not header_row and not footer_row)
+                else None
+            )
             max_height = 1
             cells: List[List[List[Segment]]] = []
             if header_row or footer_row:
-                row_style = Style.empty()
+                row_style = Style.null()
             else:
                 row_style = get_style(
-                    get_row_style(index - 1 if show_header else index)
+                    get_row_style(console, index - 1 if show_header else index)
                 )
-            for width, cell, column in zip(widths, row, columns):
+            for width, cell, column in zip(widths, row_cell, columns):
                 render_options = options.update(
                     width=width,
                     justify=column.justify,
@@ -674,7 +720,9 @@ class Table(JupyterMixin):
                 cells.append(lines)
 
             cells[:] = [
-                _Segment.set_shape(_cell, width, max_height, style=table_style)
+                _Segment.set_shape(
+                    _cell, width, max_height, style=table_style + row_style
+                )
                 for width, _cell in zip(widths, cells)
             ]
 
@@ -684,13 +732,16 @@ class Table(JupyterMixin):
                         _box.get_row(widths, "foot", edge=show_edge), border_style
                     )
                     yield new_line
-                if first:
-                    left, right, divider = box_segments[0]
-                elif last:
-                    left, right, divider = box_segments[2]
-                else:
-                    left, right, divider = box_segments[1]
+                left, right, _divider = box_segments[0 if first else (2 if last else 1)]
 
+                # If the column divider is whitespace also style it with the row background
+                divider = (
+                    _divider
+                    if _divider.text.strip()
+                    else _Segment(
+                        _divider.text, row_style.background_style + _divider.style
+                    )
+                )
                 for line_no in range(max_height):
                     if show_edge:
                         yield left
@@ -707,22 +758,22 @@ class Table(JupyterMixin):
                         yield from rendered_cell[line_no]
                     yield new_line
             if _box and first and show_header:
-                yield Segment(
+                yield _Segment(
                     _box.get_row(widths, "head", edge=show_edge), border_style
                 )
                 yield new_line
-            if _box and (show_lines or leading):
+            end_section = row and row.end_section
+            if _box and (show_lines or leading or end_section):
                 if (
                     not last
-                    and not (show_footer and index >= len(rows) - 2)
+                    and not (show_footer and index >= len(row_cells) - 2)
                     and not (show_header and header_row)
                 ):
                     if leading:
-                        for _ in range(leading):
-                            yield _Segment(
-                                _box.get_row(widths, "mid", edge=show_edge),
-                                border_style,
-                            )
+                        yield _Segment(
+                            _box.get_row(widths, "mid", edge=show_edge) * leading,
+                            border_style,
+                        )
                     else:
                         yield _Segment(
                             _box.get_row(widths, "row", edge=show_edge), border_style
@@ -735,25 +786,76 @@ class Table(JupyterMixin):
 
 
 if __name__ == "__main__":  # pragma: no cover
-    from .console import Console
+    from rich.console import Console
+    from rich.highlighter import ReprHighlighter
+    from rich.table import Table
 
-    c = Console()
     table = Table(
-        show_lines=False,
-        row_styles=["red", "green"],
-        expand=False,
-        show_header=True,
-        show_footer=False,
-        show_edge=True,
+        title="Star Wars Movies",
+        caption="Rich example table",
+        caption_justify="right",
     )
-    table.add_column("foo", no_wrap=True, footer="BAR")
-    table.add_column("bar")
-    table.add_column("baz")
-    table.add_row("Magnet", "foo" * 20, "bar" * 10, "egg" * 15)
 
-    for width in range(170, 1, -1):
-        print(" " * width + "<|")
-        c = Console(width=width)
-        c.print(table)
+    table.add_column("Released", header_style="bright_cyan", style="cyan", no_wrap=True)
+    table.add_column("Title", style="magenta")
+    table.add_column("Box Office", justify="right", style="green")
 
-    c.print("Some more words", width=4, overflow="ellipsis")
+    table.add_row(
+        "Dec 20, 2019",
+        "Star Wars: The Rise of Skywalker",
+        "$952,110,690",
+    )
+    table.add_row("May 25, 2018", "Solo: A Star Wars Story", "$393,151,347")
+    table.add_row(
+        "Dec 15, 2017",
+        "Star Wars Ep. V111: The Last Jedi",
+        "$1,332,539,889",
+        style="on black",
+        end_section=True,
+    )
+    table.add_row(
+        "Dec 16, 2016",
+        "Rogue One: A Star Wars Story",
+        "$1,332,439,889",
+    )
+
+    def header(text: str) -> None:
+        console.print()
+        console.rule(highlight(text))
+        console.print()
+
+    console = Console()
+    highlight = ReprHighlighter()
+    header("Example Table")
+    console.print(table, justify="center")
+
+    table.expand = True
+    header("expand=True")
+    console.print(table, justify="center")
+
+    table.width = 50
+    header("width=50")
+
+    console.print(table, justify="center")
+
+    table.width = None
+    table.expand = False
+    table.row_styles = ["dim", "none"]
+    header("row_styles=['dim', 'none']")
+
+    console.print(table, justify="center")
+
+    table.width = None
+    table.expand = False
+    table.row_styles = ["dim", "none"]
+    table.leading = 1
+    header("leading=1, row_styles=['dim', 'none']")
+    console.print(table, justify="center")
+
+    table.width = None
+    table.expand = False
+    table.row_styles = ["dim", "none"]
+    table.show_lines = True
+    table.leading = 0
+    header("show_lines=True, row_styles=['dim', 'none']")
+    console.print(table, justify="center")
