@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 
+import os.path
 import platform
 import sys
 from dataclasses import dataclass, field
@@ -41,6 +42,9 @@ from .theme import Theme
 
 WINDOWS = platform.system() == "Windows"
 
+LOCALS_MAX_LENGTH = 10
+LOCALS_MAX_STRING = 80
+
 
 def install(
     *,
@@ -120,6 +124,7 @@ class Stack:
     exc_type: str
     exc_value: str
     syntax_error: Optional[_SyntaxError] = None
+    is_cause: bool = False
     frames: List[Frame] = field(default_factory=list)
 
 
@@ -144,6 +149,9 @@ class Traceback:
         word_wrap (bool, optional): Enable word wrapping of long lines. Defaults to False.
         show_locals (bool, optional): Enable display of local variables. Defaults to False.
         indent_guides (bool, optional): Enable indent guides in code and locals. Defaults to True.
+        locals_max_length (int, optional): Maximum length of containers before abbreviating, or None for no abbreviation.
+            Defaults to 10.
+        locals_max_string (int, optional): Maximum length of string before truncating, or None to disable. Defaults to 80.
     """
 
     def __init__(
@@ -155,6 +163,8 @@ class Traceback:
         word_wrap: bool = False,
         show_locals: bool = False,
         indent_guides: bool = True,
+        locals_max_length: int = LOCALS_MAX_LENGTH,
+        locals_max_string: int = LOCALS_MAX_STRING,
     ):
         if trace is None:
             exc_type, exc_value, traceback = sys.exc_info()
@@ -172,6 +182,8 @@ class Traceback:
         self.word_wrap = word_wrap
         self.show_locals = show_locals
         self.indent_guides = indent_guides
+        self.locals_max_length = locals_max_length
+        self.locals_max_string = locals_max_string
 
     @classmethod
     def from_exception(
@@ -185,6 +197,8 @@ class Traceback:
         word_wrap: bool = False,
         show_locals: bool = False,
         indent_guides: bool = True,
+        locals_max_length: int = LOCALS_MAX_LENGTH,
+        locals_max_string: int = LOCALS_MAX_STRING,
     ) -> "Traceback":
         """Create a traceback from exception info
 
@@ -198,6 +212,9 @@ class Traceback:
             word_wrap (bool, optional): Enable word wrapping of long lines. Defaults to False.
             show_locals (bool, optional): Enable display of local variables. Defaults to False.
             indent_guides (bool, optional): Enable indent guides in code and locals. Defaults to True.
+            locals_max_length (int, optional): Maximum length of containers before abbreviating, or None for no abbreviation.
+                Defaults to 10.
+            locals_max_string (int, optional): Maximum length of string before truncating, or None to disable. Defaults to 80.
 
         Returns:
             Traceback: A Traceback instance that may be printed.
@@ -213,6 +230,8 @@ class Traceback:
             word_wrap=word_wrap,
             show_locals=show_locals,
             indent_guides=indent_guides,
+            locals_max_length=locals_max_length,
+            locals_max_string=locals_max_string,
         )
 
     @classmethod
@@ -222,6 +241,8 @@ class Traceback:
         exc_value: BaseException,
         traceback: Optional[TracebackType],
         show_locals: bool = False,
+        locals_max_length: int = LOCALS_MAX_LENGTH,
+        locals_max_string: int = LOCALS_MAX_STRING,
     ) -> Trace:
         """Extract traceback information.
 
@@ -230,14 +251,23 @@ class Traceback:
             exc_value (BaseException): Exception value.
             traceback (TracebackType): Python Traceback object.
             show_locals (bool, optional): Enable display of local variables. Defaults to False.
+            locals_max_length (int, optional): Maximum length of containers before abbreviating, or None for no abbreviation.
+                Defaults to 10.
+            locals_max_string (int, optional): Maximum length of string before truncating, or None to disable. Defaults to 80.
 
         Returns:
             Trace: A Trace instance which you can use to construct a `Traceback`.
         """
 
         stacks: List[Stack] = []
+        is_cause = False
+
         while True:
-            stack = Stack(exc_type=str(exc_type.__name__), exc_value=str(exc_value))
+            stack = Stack(
+                exc_type=str(exc_type.__name__),
+                exc_value=str(exc_value),
+                is_cause=is_cause,
+            )
 
             if isinstance(exc_value, SyntaxError):
                 stack.syntax_error = _SyntaxError(
@@ -252,12 +282,18 @@ class Traceback:
             append = stack.frames.append
 
             for frame_summary, line_no in walk_tb(traceback):
+                filename = frame_summary.f_code.co_filename
+                filename = os.path.abspath(filename) if filename else "?"
                 frame = Frame(
-                    filename=frame_summary.f_code.co_filename or "?",
+                    filename=filename,
                     lineno=line_no,
                     name=frame_summary.f_code.co_name,
                     locals={
-                        key: pretty.traverse(value)
+                        key: pretty.traverse(
+                            value,
+                            max_length=locals_max_length,
+                            max_string=locals_max_string,
+                        )
                         for key, value in frame_summary.f_locals.items()
                     }
                     if show_locals
@@ -265,12 +301,26 @@ class Traceback:
                 )
                 append(frame)
 
-            cause = exc_value.__context__
+            cause = getattr(exc_value, "__cause__", None)
             if cause and cause.__traceback__:
                 exc_type = cause.__class__
                 exc_value = cause
                 traceback = cause.__traceback__
                 if traceback:
+                    is_cause = True
+                    continue
+
+            cause = exc_value.__context__
+            if (
+                cause
+                and cause.__traceback__
+                and not getattr(exc_value, "__suppress_context__", False)
+            ):
+                exc_type = cause.__class__
+                exc_value = cause
+                traceback = cause.__traceback__
+                if traceback:
+                    is_cause = False
                     continue
             # No cover, code is reached but coverage doesn't recognize it.
             break  # pragma: no cover
@@ -344,9 +394,14 @@ class Traceback:
                 )
 
             if not last:
-                yield Text.from_markup(
-                    "\n[i]During handling of the above exception, another exception occurred:\n",
-                )
+                if stack.is_cause:
+                    yield Text.from_markup(
+                        "\n[i]The above exception was the direct cause of the following exception:\n",
+                    )
+                else:
+                    yield Text.from_markup(
+                        "\n[i]During handling of the above exception, another exception occurred:\n",
+                    )
 
     @render_group()
     def _render_syntax_error(self, syntax_error: _SyntaxError) -> RenderResult:
@@ -436,6 +491,8 @@ class Traceback:
                                 frame.locals,
                                 title="locals",
                                 indent_guides=self.indent_guides,
+                                max_length=self.locals_max_length,
+                                max_string=self.locals_max_string,
                             ),
                         ],
                         padding=1,
